#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import os
import markdown

from acrylamid.filters import Filter
from acrylamid.filters import log

class Markdown(Filter):
    
    __name__ = 'Markdown'
    __match__ = ['md', 'mkdown', 'markdown', 'Markdown']
    __conflicts__ = ['rst', 'plain']
    __priority__ = 70.0
    
    __ext__ = dict((x,x) for x in ['abbr', 'fenced_code', 'footnotes',
                                   'headerid', 'tables', 'codehilite'])

    def __init__(self, conf, env):
        
        self.env = env
        # -- discover markdown extensions --
        
        for mem in os.listdir(os.path.dirname(__file__)):
            if mem.startswith('mdx_') and mem.endswith('.py'):
                try:
                    mod = __import__(mem.replace('.py', ''))
                    mdx = mod.makeExtension()
                    if isinstance(mod.__match__, basestring):
                        self.__match__.append(mod.__match__)
                        self.__ext__[mod.__name__] = mdx
                    else:
                        for name in mod.__match__:
                            self.__ext__[name] = mdx
                except (ImportError, Exception), e:
                    print `mem`, 'ImportError:', e

    def __contains__(self, key):
        return True if key in self.__ext__ else False
        
    def __call__(self, content, request, *filters):
        
        err = []; val = []
        for f in filters:
            if f in self:
                val.append(f)
            else:
                x = f.split('(', 1)[:1][0]
                if not x in self:
                    log.warn('%s is not available' % x)
                else:
                    val.append(x)
                    self.__ext__[x] = f

        md = markdown.Markdown(extensions=[self.__ext__[m] for m in val])
        return md.convert(content)



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

from acrylamid.filters import Filter
from re import sub

class Headoffset(Filter):
    
    __name__ = 'head_offset'
    __match__ = ['h'+str(i+1) for i in range(5)]
    
    def __init__(self, conf, env):
        pass
        
    def __call__(self, content, request, *args):
        
        def f(m):
            '''will return html with all headers increased by 1'''
            l = lambda i: i if i not in [str(x) for x in range(1, 6)] else str(int(i)+1) if int(i) < 5 else '6'
            return ''.join([l(i) for i in m.groups()])

        offset = int(self.__matched__[1])
        
        for i in range(offset):
            content = sub('(<h)(\d)(>)(.+)(</h)(\d)(>)', f, content)

        return content



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

from os.path import exists

from acrylamid.views import View
from acrylamid.utils import expand, render, mkfile, joinurl, event

filters = []
path = '/:year/:slug/'
enabled = True

class Entry(View):
    """Creates single full-length entry.
    entry.html -- layout of Post's entry
    main.html -- layout of the website
    """
    
    def __init__(self, conf, env):
        pass
        
    def __call__(self, request):

        conf = request['conf']
        env = request['env']
        entrylist = request['entrylist']
        
        tt_entry = env['tt_env'].get_template('entry.html')
        tt_main = env['tt_env'].get_template('main.html')

        for entry in entrylist:
            if entry.permalink != expand(path, entry):
                p = joinurl(conf['output_dir'], entry.permalink, 'index.html')
            else:
                p = joinurl(conf['output_dir'], expand(path, entry), 'index.html')
            
            if exists(p) and not entry.has_changed:
                if not (tt_entry.has_changed or tt_main.has_changed):
                    event.skip(entry.title)
                    continue
            
            html = render(tt_main, conf, env, type='item',
                          entrylist=render(tt_entry, conf, env, entry, type='item'),
                          title=entry.title, description=entry.description, tags=entry.tags)
            
            mkfile(html, p, entry.title)



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py
# -*- encoding: utf-8 -*-

from os.path import exists

from acrylamid.views import View
from acrylamid.utils import render, mkfile, joinurl, event, paginate

filters = []
path = '/page/'
items_per_page = 10
paginating = True
enabled = True

class Index(View):
    
    def __init__(self, conf, env):
        pass
        
    def __call__(self, request):
        """Creates nicely paged listing of your posts.  First page is the
        index.hml used to have this nice url: http://yourblog.com/ with a recent
        list of your (e.g. summarized) Posts. Other pages are enumerated to /page/n+1

        required:
        items_per_page -- posts displayed per page (defaults to 6)
        entry.html -- layout of Post's entry
        main.html -- layout of the website
        """
        conf = request['conf']
        env = request['env']
        entrylist = request['entrylist']
        ipp = items_per_page

        tt_entry = env['tt_env'].get_template('entry.html')
        tt_main = env['tt_env'].get_template('main.html')

        for i, mem in enumerate(paginate(entrylist, items_per_page, lambda e: not e.draft)):

            if i == 0:
                next = None
                curr = path
            else:
                curr = joinurl(path, i+1)
                next = '/' if i==1 else joinurl(path, i)
            
            prev = None if i==(len(entrylist)/ipp+1)-1 else joinurl(path, i+2)
            directory = conf['output_dir']
            
            if i > 0:
                directory = joinurl(conf['output_dir'], (path + str(i+1)))                
            p = joinurl(directory, 'index.html')
            message = '/' if i==0 else (path+str(i+1))
            
            if exists(p) and not filter(lambda e: e.has_changed, mem):
                if not (tt_entry.has_changed or tt_main.has_changed):
                    event.skip(message)
                    continue
            
            mem = [render(tt_entry, conf, env, entry, type="page") for entry in mem]
            
            html = render(tt_main, conf, env, type='page', prev=prev, curr=curr, next=next,
                        entrylist='\n'.join(mem), num_entries=len(entrylist),
                        items_per_page=items_per_page)
            
            mkfile(html, p, message)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import logging
import os
import fnmatch
import time
from datetime import datetime

from acrylamid.utils import FileEntry
log = logging.getLogger('acrylamid.core')
    

def handle(request):
    """This will prepare the whole thing. Dir-walks through content dir and
    try to get the timestamp (fallback to '%d.%m.%Y, %H:%M' parsing, or even
    worse: mtime). return entrylist reverse sorted by date."""
        
    conf = request['conf']

    # generate a list of entries
    request['entrylist'] = filelist(request)

    for i,entry in enumerate(request['entrylist']):
        # convert mtime timestamp or `date:` to localtime (float), required for sort
        if isinstance(entry.date, basestring):
            timestamp = time.mktime(time.strptime(entry.date,
                conf.get('strptime', '%d.%m.%Y, %H:%M')))
            entry.date = datetime.fromtimestamp(timestamp)
        else:
            log.warn("using mtime from %s" % entry)
    
    request['entrylist'].sort(key=lambda k: k.date, reverse=True)
    return request


def filelist(request):
    """gathering all entries in entries_dir except entries_ignore via fnmatch."""
    
    conf = request['conf']
    
    filelist = []
    for root, dirs, files in os.walk(conf['entries_dir']):
        for file in files:
            path = os.path.join(root, file)
            fn = filter(lambda p: fnmatch.fnmatch(path, os.path.join(conf['entries_dir'], p)),
                        conf.get('entries_ignore', []))
            if not fn:
                filelist.append(path)
    
    entrylist = [FileEntry(e, encoding=conf['encoding']) for e in filelist]
    return entrylist



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

from acrylamid.views import View
from acrylamid.utils import mkfile, joinurl

from os.path import join, getmtime, exists
from collections import defaultdict
from jinja2 import Template

filters = []
path = '/articles/'
enabled = True

class Articles(View):
    """Generates a overview of all articles."""
    
    __filters__ = False
    
    def __init__(self, conf, env):
        self.layoutpath = join(conf['layout_dir'], 'articles.html')
        with file(self.layoutpath) as f:
            self.tt_articles = Template(f.read())
        
    def __call__(self, request):
        
        articles = defaultdict(list)
        conf = request['conf']
        entrylist = request['entrylist']
        
        p = joinurl(conf['output_dir'], path, 'index.html')
        last_modified = max([getmtime(e.filename) for e in entrylist])
        
        if exists(p) and last_modified < getmtime(p):
            if getmtime(self.layoutpath) < getmtime(p):
                return
        
        for entry in sorted(entrylist, key=lambda k: k.date, reverse=True):
            if entry.draft: continue

            url, title, year = entry.permalink, entry.title, entry.date.year
            articles[year].append((entry.date, url, title))
        
        articlesdict = conf.copy()
        articlesdict.update({'articles': articles,
                     'num_entries': len(entrylist)})
                 
        html = self.tt_articles.render(articlesdict)
        mkfile(html, p, joinurl(path, 'index.html'))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import sys
import os
import logging
from acrylamid.utils import yamllike

from os.path import exists, isfile, isdir, join

log = logging.getLogger('acrylamid.defaults')

def init(root='.', overwrite=False):
    
    dirs = ['%(entries_dir)s/', '%(layout_dir)s/', '%(output_dir)s/']
    files = {'conf.yaml': conf,
             '%(output_dir)s/blog.css': css,
             '%(layout_dir)s/main.html': main,
             '%(layout_dir)s/entry.html': entry,
             '%(layout_dir)s/articles.html': articles,
             '%(entries_dir)s/sample entry.txt': kafka}
             
    default = yamllike(conf)
    default['output_dir'] = default.get('output_dir', 'output/').rstrip('/')
    default['entries_dir'] = default.get('entries_dir', 'content/').rstrip('/')
    default['layout_dir'] = default.get('layout_dir', 'layouts/').rstrip('/')
    
    if root != '.' and not exists(root):
        os.mkdir(root)
                
    for directory in dirs:
        directory = join(root, directory % default)
        if exists(directory) and not isdir(directory):
            log.critical('Unable to create %s. Please remove this file', directory)
            sys.exit(1)
        elif not exists(directory):
            os.mkdir(directory)
            log.info('create  %s', directory)
        else:
            log.info('skip  %s already exists', directory)
    
    for path, content in files.iteritems():
        path = join(root, path % default)
        if exists(path) and not isfile(path):
            log.critical('%s must be a regular file' % path)
            sys.exit(1)
        elif not exists(path) or overwrite == True:
            f = open(path, 'w')
            f.write(content)
            f.close()
            log.info('create  %s', path)
        else:
            log.info('skip  %s already exists', path)


conf =  '''
blog_title: A descriptive blog title

author: anonymous
website: http://example.org/
email: info@example.org

www_root: http://example.org/
lang: de_DE
strptime: %d.%m.%Y, %H:%M
encoding: utf-8

disqus_shortname: yourname

views.filters: ['markdown+codehilite(css_class=highlight)', 'hyphenate']

views.index.filters: ['summarize', 'h1']
views.entry.filters: ['h1']
views.feeds.filters: ['h2']
'''.strip()

css = '''
@import url(pygments.css);
body {
  background-color: white; }

#blogheader {
  margin-bottom: 10px;
  padding: 10px;
  font-family: Palatino, "Times New Roman", serif;
  text-decoration: none;
  text-align: center; }
  #blogheader #blogtitle h2 {
    padding-top: 1.5em; }
    #blogheader #blogtitle h2 a {
      color: black;
      text-decoration: none; }
  #blogheader #mainlinks {
    padding: 1em; }
    #blogheader #mainlinks li {
      display: inline-block;
      margin: 0 2em 0 2em; }
    #blogheader #mainlinks a {
      color: black;
      text-decoration: none;
      font-family: Palatino, "Times New Roman", serif; }
      #blogheader #mainlinks a:hover {
        text-shadow: #888888 0px 0px 1px; }

#blogbody {
  margin: 0 auto;
  width: 800px; }
  #blogbody .posting {
    padding: 1em;
    margin-top: 64px;
    margin-bottom: 64px; }
    #blogbody .posting .postheader .subject {
      margin-bottom: -0.7em; }
      #blogbody .posting .postheader .subject a {
        color: black;
        font: bold medium Palatino, "Times New Roman";
        text-decoration: none; }
        #blogbody .posting .postheader .subject a:hover {
          text-shadow: #aaaaaa 0px 0px 2px; }
    #blogbody .posting .postheader .date {
      font: 0.7em "Georgia";
      float: right; }
    #blogbody .posting .postbody {
      text-align: justify;
      padding: 1em;
      font-family: Palatino, "Times New Roman", serif;
      font-size: 11pt; }
      #blogbody .posting .postbody h2 {
        font: 12pt Palatino, "Times New Roman", serif;
        font-weight: bold;
        color: #888888;
        border-bottom: 1px dotted #888888; }
      #blogbody .posting .postbody h3, #blogbody .posting .postbody h4 {
        font: 12pt Palatino, "Times New Roman", serif;
        font-weight: bold;
        color: #888888;
        padding-left: 4px; }
      #blogbody .posting .postbody a {
        text-decoration: none;
        color: #cc0000; }
        #blogbody .posting .postbody a:visited {
          color: #aa0000; }
        #blogbody .posting .postbody a:hover {
          color: #ff190d;
          text-shadow: #ff6666 0px 0px 1px; }
      #blogbody .posting .postbody p {
        line-height: 1.3em; }
      #blogbody .posting .postbody dl {
        padding-left: 16px; }
        #blogbody .posting .postbody dl dt {
          font-weight: bold;
          color: #444444;
          padding: 6px 92px 3px 0;
          padding-top: 9px; }
      #blogbody .posting .postbody ul {
        padding: 10px 0 10px 40px;
        list-style-type: disc; }
      #blogbody .posting .postbody li {
        padding-top: 3px; }
      #blogbody .posting .postbody pre, #blogbody .posting .postbody code {
        font-family: Bitstream Vera Sans Mono, monospace;
        font-size: 13px; }
      #blogbody .posting .postbody blockquote {
        border-left: 3pt solid #aaaaaa;
        padding-left: 1em;
        padding-right: 1em;
        margin-left: 1em;
        font: italic small Verdana, Times, sans-serif;
        color: #222222; }
        #blogbody .posting .postbody blockquote em {
          font-weight: bold; }
      #blogbody .posting .postbody img {
        max-width: 700px;
        margin: 0em 20px;
        -moz-box-shadow: 0px 0px 9px black;
        -webkit-box-shadow: 0px 0px 9px black;
        box-shadow: 0px 0px 4px black; }
      #blogbody .posting .postbody .amp {
        color: #666666;
        font-family: "Warnock Pro", "Goudy Old Style", "Palatino", "Book Antiqua", serif;
        font-style: italic; }
      #blogbody .posting .postbody .caps {
        font-size: 0.92em; }
      #blogbody .posting .postbody .highlight {
        border: 1px solid #cccccc;
        padding-left: 1em;
        margin-bottom: 10px;
        margin-top: 10px;
        background: none repeat scroll 0 0 #f0f0f0;
        overflow: auto; }
    #blogbody .posting .postfooter p {
      margin: 0 0 0 0;
      font-style: italic; }
    #blogbody .posting .postfooter a {
      color: #111111;
      font-family: Palatino, "times new roman", serif;
      font-weight: bold;
      font-size: 0.9em;
      text-decoration: none; }
      #blogbody .posting .postfooter a:hover {
        color: #111111;
        text-shadow: #aaaaaa 0px 0px 1px; }
  #blogbody .page {
    margin-top: -20px;
    font: bold medium Palatino, "Times New Roman", serif;
    color: black;
    text-decoration: none;
    border-bottom: 1px dotted black; }
    #blogbody .page:hover {
      text-shadow: #aaaaaa 0px 0px 1px; }

#blogfooter {
  margin-top: 10px;
  padding: 10px;
  text-align: center;
  font: medium Palatino, "Times New Roman", serif; }
  #blogfooter a {
    font-weight: bold;
    text-decoration: none;
    border-bottom: 1px dotted black;
    color: #cc0000; }
    #blogfooter a:hover {
      text-shadow: #ee6688 0px 0px 1px;
      color: #ff190d; }

#disqus_thread {
  padding-top: 24px; }

object[type="application/x-shockwave-flash"] {
  -moz-box-shadow: 0px 0px 12px #777777;
  -webkit-box-shadow: 0px 0px 12px #777777;
  box-shadow: 0px 0px 12px #777777;
  margin-left: 15px;
  text-align: center; }

.shadow {
  -moz-box-shadow: 0px 0px 9px black;
  -webkit-box-shadow: 0px 0px 9px black;
  box-shadow: 0px 0px 4px black; }

.floatright {
  float: right; }

.floatleft {
  float: left; }
'''.strip()

main = r'''
<!DOCTYPE html
  PUBLIC "-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN"
         "http://www.w3.org/Math/DTD/mathml2/xhtml-math11-f.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
  <title>
      {%- if type != 'item' -%}
        {{ blog_title }}
      {%- else -%}
        {{ title }}
      {%- endif -%}
  </title>
  <meta http-equiv="Content-Type" content="text/xhtml; charset=utf-8" />
  <meta http-equiv="content-language" content="de, en" />
  {% if type == 'item' -%}
  <meta name="description" content="{{ description }}" />
  <meta name="keywords" content="{{ tags | join(', ') }}" />
  {%- endif %}
  <link media="all" href="/blog.css" type="text/css" rel="stylesheet" />
  <link href="/favicon.ico" rel="shortcut icon" />
  <link href="/" rel="home" />
  <link href="/atom/" type="application/atom+xml" rel="alternate" title="Atom-Feed" />
  <link href="/rss/" type="application/rss+xml" rel="alternate" title="RSS-Feed" />
 </head>
 <body>
    <div id="blogheader">
        <div id="blogtitle">
            <h2>
                <a href="/" class="blogtitle">{{ blog_title }}</a>
            </h2>
        </div>
        <div id="mainlinks">
            <ul>
                <li><a href="/">blog</a></li>
                <li><a href="/atom/">atom</a></li>
                <li><a href="/rss/">rss</a></li>
                <li><a href="/articles/">articles</a></li>
            </ul>
        </div>
    </div>
    <div id="blogbody">
        {{ entrylist }}
        {% if type in ['tag', 'page'] %}
            {% if prev %}
                <a href="{{ prev }}" class="page floatright">
                ältere Beiträge →
                </a>
            {% endif %}
            {% if next %}
                <a href="{{ next }}" class="page floatleft">
                ← neuere Beiträge
                </a>
            {% endif %}
        {%- endif  %}
    </div>
    <div id="blogfooter">
        <p>
            written by <a href="mailto:{{ email }}">{{ author }}</a>
        </p>
        <a href="http://creativecommons.org/licenses/by-nc-sa/2.0/de/">
            <img src="/img/cc.png" alt="by-nc-sa" />
        </a>
    </div>
    {% if disqus_shortname and type == 'page' %}
    <script type="text/javascript">
        /* * * CONFIGURATION VARIABLES: EDIT BEFORE PASTING INTO YOUR WEBPAGE * * */
        var disqus_shortname = '{{ disqus_shortname }}'; // required: replace example with your forum shortname

        /* * * DON'T EDIT BELOW THIS LINE * * */
        (function () {
            var s = document.createElement('script'); s.async = true;
            s.type = 'text/javascript';
            s.src = '{{ protocol }}://' + disqus_shortname + '.disqus.com/count.js';
            (document.getElementsByTagName('HEAD')[0] || document.getElementsByTagName('BODY')[0]).appendChild(s);
        }());
    </script>
    {% endif %}
 </body>
</html>'''.strip()

entry = r'''
<div class="posting">
    <div class="postheader">
        <h1 class="subject">
            <a href="{{ permalink }}">{{ title }}</a>
        </h1>
        <span class="date">{{ date.strftime("%d.%m.%Y, %H:%M") }}</span>
    </div>
    <div class="postbody">        
        {{ content }}
    </div>
    <div class="postfooter">
        {% if disqus_shortname and type == 'page' %}
            <a class="floatright" href="{{ www_root + permalink }}#disqus_thread">Kommentieren</a>
        {% endif %}
        {% if tags %}
            <p>verschlagwortet als
                {% for link in tags | tagify -%}
                    <a href="{{ link.href }}">{{ link.title }}</a>
                    {%- if loop.revindex > 2 -%}
                    ,
                    {%- elif loop.revindex == 2 %}
                    und 
                    {% endif %}
                {% endfor %}
            </p>
        {% else %}
            <p>nicht verschlagwortet</p>
        {% endif %}
    </div>
    <div class="comments">
        {% if disqus_shortname and type == 'item' %}
        <div id="disqus_thread"></div>
        <script type="text/javascript">
            var disqus_shortname = '{{ disqus_shortname }}'; // required: replace example with your forum shortname

            // The following are highly recommended additional parameters. Remove the slashes in front to use.
            var disqus_identifier = "{{ www_root + permalink }}";
            var disqus_url = "{{ www_root + permalink }}";

            /* * * DON'T EDIT BELOW THIS LINE * * */
            (function() {
                var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
                dsq.src = '{{ protocol }}://' + disqus_shortname + '.disqus.com/embed.js';
                (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
            })();
        </script>
        <noscript>
            <p>Please enable JavaScript to view the <a href="{{ protocol }}://disqus.com/?ref_noscript">comments powered by Disqus.</a></p>
        </noscript>
        <a href="{{ protocol }}://disqus.com" class="dsq-brlink">
            blog comments powered by <span class="logo-disqus">Disqus</span>
        </a>
        {% endif %}
    </div>
</div>'''.strip()

articles = r'''
<xhtml>
    <head>
        <title>{{ blog_title }} – Artikelübersicht</title>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <link rel="stylesheet" type="text/css" media="all," href="/blog.css" />
        <link rel="shortcut icon" href="/favicon.ico" />
        <link rel="name" href="/" />
        <link rel="alternate" title="Atom-Feed" type="application/atom+xml" href="/atom/" />
        <link rel="alternate" title="RSS-Feed" type="application/rss+xml" href="/rss/" />
    </head>
    <body>
        <div id="blogheader">
            <div id="blogtitle">
                <h2>
                    <a href="/" class="blogtitle">{{ blog_title }}</a>
                </h2>
            </div>
            <div id="mainlinks">
                <ul>
                    <li>
                        <a href="/">blog</a>
                    </li>
                    <li>
                        <a href="/atom/">atom</a>
                    </li>
                    <li>
                        <a href="/rss/">rss</a>
                    </li>
                    <li>
                        <a href="/articles/">articles</a>
                    </li>
                </ul>
            </div>
        </div>
        <div id="blogbody">
            <div class="posting">
                <div class="postheader">
                    <span class="date">{{ num_entries }} Beiträge</span>
                </div>
                <div class="postbody">
                    {% for year in articles|sort(reverse=True) %}
                    <h2>{{ year }}</h2>
                    <ul>
                        {% for entry in articles[year] %}
                            <li>
                                <span>{{ entry[0].strftime('%d.%m.%Y: ') }}</span>
                                <a href="{{entry[1]}}">{{ entry[2] }}</a>
                            </li>
                        {% endfor %}
                    </ul>
                    {% endfor %}
                </div>
            </div>
        </div>
        <div id="blogfooter">
            <p>
                written by
                <a href="mailto:{{ email }}">{{ author }}</a>
            </p>
            <a href="http://creativecommons.org/licenses/by-nc-sa/2.0/de/">
                <img src="/img/cc.png" alt="by-nc-sa" />
            </a>
        </div>
    </body>
</xhtml>
'''

kafka = '''
---
title: Die Verwandlung
author: Franz Kafka
tags: [Franz Kafka, Die Verwandlung]
---

Als Gregor Samsa eines Morgens aus unruhigen Träumen erwachte, fand er sich in
seinem Bett zu einem ungeheueren Ungeziefer verwandelt. Er lag auf seinem
panzerartig harten Rücken und sah, wenn er den Kopf ein wenig hob, seinen
gewölbten, braunen, von bogenförmigen Versteifungen geteilten Bauch, auf
dessen Höhe sich die Bettdecke, zum gänzlichen Niedergleiten bereit, kaum noch
erhalten konnte. Seine vielen, im Vergleich zu seinem sonstigen Umfang
kläglich dünnen Beine flimmerten ihm hilflos vor den Augen.

»Was ist mit mir geschehen?« dachte er. Es war kein Traum, sein Zimmer, ein
richtiges, nur etwas zu kleines Menschenzimmer, lag ruhig zwischen den vier
wohlbekannten Wänden, über dem Tisch, auf dem eine auseinandergepackte
Musterkollektion von Tuchwaren ausgebreitet war -- Samsa war Reisender --, hing
das Bild, das er vor kurzem aus einer illustrierten Zeitschrift ausgeschnitten
und in einem hübschen, vergoldeten Rahmen untergebracht hatte. Es stellte eine
Dame dar, die, mit einem Pelzhut und einer Pelzboa versehen, aufrecht dasaß
und einen schweren Pelzmuff, in dem ihr ganzer Unterarm verschwunden war, dem
Beschauer entgegenhob.'''.strip()



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py
# -*- encoding: utf-8 -*-

from datetime import datetime
from os.path import exists

from acrylamid.views import View
from acrylamid.utils import render, mkfile, joinurl, event

from jinja2 import Environment

filters = []
enabled = True

class Feed(View):
        
    num_entries = 25
    env = Environment()
    
    def __call__(self, request):
        
        conf = request['conf']
        env = request['env']
        entrylist = request['entrylist']
        p = joinurl(conf['output_dir'], self.path , 'index.html')
        
        if exists(p) and not filter(lambda e: e.has_changed, entrylist[:self.num_entries]):
            event.skip(joinurl(self.path, 'index.html'))
            return
        
        result = []
        for entry in entrylist[:self.num_entries]:
            if entry.draft: continue
            _id = 'tag:' + conf.get('www_root', '').replace(env['protocol']+'://', '') \
                         + ',' + entry.date.strftime('%Y-%m-%d') + ':' \
                         + entry.permalink
            result.append(render(self.tt_entry, env, conf, entry, id=_id))
        
        xml = render(self.tt_body, env, conf, {'entrylist': '\n'.join(result),
                      'updated': entrylist[0].date if entrylist else datetime.now()},
                      atom=atom, rss=rss)
        
        mkfile(xml, p, joinurl(self.path, 'index.html'))

        
class atom(Feed):
    
    __view__ = True
    path = '/atom/'

    def __init__(self, conf, env):
        self.tt_entry = self.env.from_string(ATOM_ENTRY)
        self.tt_body = self.env.from_string(ATOM_BODY)


class rss(Feed):

    __view__ = True
    path = '/rss/'

    def __init__(self, conf, env):      
        
        from wsgiref.handlers import format_date_time
        from time import mktime
        
        self.env.filters['rfc822'] = lambda x: format_date_time(mktime(x.timetuple()))
        self.tt_entry = self.env.from_string(RSS_ENTRY)
        self.tt_body = self.env.from_string(RSS_BODY)


ATOM_BODY = r'''
<?xml version="1.0" encoding="utf-8"?>
<feed xmlns="http://www.w3.org/2005/Atom" xml:lang="{{ lang[0].replace('_', '-') }}">
    <author>
        <name>{{ author }}</name>
        <uri>{{ website }}</uri>
        <email>{{ email }}</email>
    </author>
    <title>{{ blog_title | escape }}</title>
    <id>{{ website }}</id>
    <link rel="alternate" type="text/html" href="{{ website }}" />
    <link rel="self" type="application/atom+xml" href="{{ www_root+atom.path }}" />
    <updated>{{ updated.strftime('%Y-%m-%dT%H:%M:%SZ') }}</updated>
    <generator uri="https://posativ.org/acrylamid/" version="{{ acrylamid_version }}">{{ acrylamid_name }}</generator>
    {{ entrylist }}
</feed>
'''.strip()

ATOM_ENTRY = r'''
<entry>
    <title>{{ title | escape }}</title>
    <link rel="alternate" type="text/html" href="{{ www_root + permalink }}" />
    <id>{{ id.rstrip('/') }}</id>
    <updated>{{ date.strftime('%Y-%m-%dT%H:%M:%SZ') }}</updated>
    <author>
        <name>{{ author }}</name>
        <uri>{{ website }}</uri>
        <email>{{ email }}</email>
    </author>
    <content type="html">{{ content | escape }}</content>
</entry>
'''.strip()

RSS_BODY = r'''
<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">
    <channel>
        <title>{{ blog_title | escape }}</title>
        <link>{{ website }}</link>
        <description>{{ description }}</description>
        <language>{{ lang[0].replace('_', '-').lower() }}</language>
        <pubDate>{{ updated | rfc822 }}</pubDate>
        <docs>{{ www_root+rss.path }}</docs>
        <generator>{{ acrylamid_name }} {{ acrylamid_version }}</generator>
        <atom:link href="{{ www_root+atom.path }}" rel="self" type="application/rss+xml" />
        {{ entrylist }}
    </channel>
</rss>
'''.strip()

RSS_ENTRY = r'''
<item>
    <title>{{ title | escape }}</title>
    <link>{{ www_root + permalink }}</link>
    <description>{{ content | escape }}</description>
    <pubDate>{{ date | rfc822 }}</pubDate>
    <guid isPermaLink="false">{{ id.rstrip('/') }}</guid>
</item>
'''.strip()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import sys, os, glob, fnmatch, copy
import logging

sys.path.insert(0, os.path.dirname(__file__))
log = logging.getLogger('acrylamid.filters')

def get_filters():
    
    global callbacks
    return callbacks


def index_filters(module, conf, env):
    """Goes through the modules' contents and indexes all the funtions/classes
    having a __call__ and __match__ attribute.
    
    :param module: the module to index
    :param conf: user config
    :param env: environment
    """
    
    global callbacks
    
    cs = [getattr(module, c) for c in dir(module) if not c.startswith('_')]
    for mem in cs:
        if callable(mem) and hasattr(mem, '__match__'):
            callbacks[getattr(mem, '__match__')] = mem(conf, env)

            
def initialize(ext_dir, conf, env, include=[], exclude=[]):
    """Imports and initializes extensions from the directories in the list
    specified by 'ext_dir'.  If no such list exists, the we don't load any
    plugins. 'include' and 'exclude' may contain a list of shell patterns
    used for fnmatch. If empty, this filter is not applied.
    
    :param ext_dir: list of directories
    :conf: user config
    :env: environment
    :param include: a list of filename patterns to include
    :param exclude: a list of filename patterns to exclude
    """
    
    def get_name(filename):
        """Takes a filename and returns the module name from the filename."""
        return os.path.splitext(os.path.split(filename)[1])[0]
    
    def get_extension_list(ext_dir, include, exclude):
        """Load all plugins that matches include/exclude pattern in
        given lust of directories.  Arguments as in initializes(**kw)."""
        
        def pattern(name, incl, excl):
            for i in incl:
                if fnmatch.fnmatch(name, i): return True
            for e in excl:
                if fnmatch.fnmatch(name, e): return False
            if not incl:
                return True
            else:
                return False
        
        ext_list = []
        for mem in ext_dir:
            files = glob.glob(os.path.join(mem, "*.py"))
            files = [get_name(f) for f in files
                                    if pattern(get_name(f), include, exclude)]
            ext_list += files
        
        return sorted(ext_list)
    
    global plugins
    
    exclude.extend(['mdx_*', 'rstx_*'])
    ext_dir.extend([os.path.dirname(__file__)])
    
    for mem in ext_dir[:]:
        if os.path.isdir(mem):
            sys.path.insert(0, mem)
        else:
            ext_dir.remove(mem)
            log.error("Extension directory '%s' does not exist. -- skipping" % mem)
            
    ext_list = get_extension_list(ext_dir, include, exclude)    
    for mem in ext_list:
        try:
            _module = __import__(mem)
        except (ImportError, Exception), e:
            print `mem`, 'ImportError:', e
            continue
        
        index_filters(_module, conf, env)


class Filter:
    
    __priority__ = 50.0
    __conflicts__ = []


class FilterList(list):
    """a list containing tuples of (filtername, filterfunc, arguments).
    
    >>> x = FilterList()
    >>> x.append((x,y,z))
    
    :param x: the filter's name, type: str
    :param y: function (Filter with __call__)
    :param z: additional arguments, a tuple of strings e.g. ('mathml', 'pygments')
    """

    def __contains__(self, y):
        """first checks, wether the item itself is in the list. Next, all Filters
        providing __conflicts__ are checked wether y conflicts with a filter.
        Otherwise y is not in FilterList.
        """
        for x in self:
            if x[1].__name__ == y.__name__:
                return True
        for f in y.__conflicts__:
            for x in self:
                if f in x[1].__match__:
                    return True
        return False
    
    def __iter__(self):
        for item in sorted(list.__iter__(self), key=lambda k: k[1].__priority__, reverse=True):
            yield item


class FilterStorage(dict):
    """store multiple keys per value and make __call__ do nothing, when
    filter is prefixed by *no*."""

    def __init__(self, *args):
        dict.__init__(self, args)
        self.map = {}

    def __contains__(self, key):
        return True if key in self.map else False

    def __setitem__(self, key, value):

        if isinstance(key, basestring):
            self.map[key] = key
            dict.__setitem__(self, key, value)
        else:
            for k in key:
                self.map[k] = key[0]
            dict.__setitem__(self, key[0], value)

    def __getitem__(self, key):

        q = key[2:] if key.startswith('no') else key
        f = dict.__getitem__(self, self.map[q])
        if key.startswith('no'):
            f = copy.copy(f)
            f.__call__ = lambda x,y,*z: x
        return f
        
callbacks = FilterStorage()



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py
# 
# -*- encoding: utf-8 -*-

from os.path import exists
from collections import defaultdict

from acrylamid.views import View
from acrylamid.utils import render, mkfile, joinurl, safeslug, event, paginate

filters = []
path = '/tag/'
items_per_page = 10
enabled = True

class Tag(View):
    
    def __init__(self, conf, env):
        
        class Link:
            
            def __init__(self, title, href):
                self.title = title
                self.href = href if href.endswith('/') else href + '/'
        
        env['tt_env'].filters['safeslug'] = safeslug
        env['tt_env'].filters['tagify'] = lambda e: [Link(t, joinurl(path, safeslug(t))) for t in e]
        
    def __call__(self, request):
        """Creates paged listing by tag.
                
        required:
        items_per_page -- posts displayed per page (defaults to 10)
        entry.html -- layout of Post's entry
        main.html -- layout of the website
        """
        conf = request['conf']
        env = request['env']
        entrylist = request['entrylist']
        ipp = items_per_page
        
        tt_entry = env['tt_env'].get_template('entry.html')
        tt_main = env['tt_env'].get_template('main.html')

        tags = defaultdict(list)
        for e in entrylist:
            for tag in e.tags:
                tags[safeslug(tag)].append(e)
        
        for tag in tags:
            entrylist = [entry for entry in tags[tag]]
            for i, mem in enumerate(paginate(entrylist, items_per_page, lambda e: not e.draft)):
                if i == 0:
                    next = None
                    curr = joinurl(path, tag)
                else:
                    curr = joinurl(path, tag, i+1)
                    next = joinurl(path, tag) if i==1 else joinurl(path, tag, i)
                
                prev = None if i==(len(entrylist)/ipp+1)-1 else joinurl(path, tag, i+2)
                p = joinurl(conf['output_dir'], curr, 'index.html')
                
                if exists(p) and not filter(lambda e: e.has_changed, mem):
                    if not (tt_entry.has_changed or tt_main.has_changed):
                        event.skip(curr)
                        continue

                mem = [render(tt_entry, conf, env, entry, type="tag") for entry in mem]
                html = render(tt_main, conf, env, type='tag', prev=prev, curr=curr, next=next,
                            entrylist='\n'.join(mem), num_entries=len(entrylist),
                            items_per_page=items_per_page)
                
                mkfile(html, p, curr)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import sys, os, glob
import logging

sys.path.insert(0, os.path.dirname(__file__))
log = logging.getLogger('acrylamid.views')

filters = []
callbacks = []

def get_views():
    
    global callbacks
    return [cb for cb in callbacks if getattr(sys.modules['acrylamid.views'], cb.__module__).enabled]


def index_views(module, conf, env):
    """Goes through the modules' contents and indexes all the funtions/classes
    having a __init__, __call__ and __match__ attribute."""

    global callbacks
    
    if not hasattr(module, 'enabled'):
        raise ImportError("'enabled' property is missing")

    cs = [getattr(module, c) for c in dir(module) if not c.startswith('_')]
    for mem in cs:
        if hasattr(mem, '__view__') and getattr(mem, '__view__') \
        and hasattr(mem, '__call__') and hasattr(mem, '__init__'):
            callbacks.append(mem(conf, env))


def initialize(ext_dir, conf, env):
    
    global callbacks

    # handle ext_dir
    for mem in ext_dir[:]:
        if os.path.isdir(mem):
            sys.path.insert(0, mem)
        else:
            ext_dir.remove(mem)
            log.error("Extension directory '%s' does not exist. -- skipping" % mem)

    ext_dir.extend([os.path.dirname(__file__)])
    ext_list = []
    for mem in ext_dir:
        files = glob.glob(os.path.join(mem, "*.py"))
        ext_list += files
    
    for mem in [os.path.basename(x).replace('.py', '') for x in ext_list]:
        if mem.startswith('_'):
            continue
        try:
            _module = __import__(mem)
            sys.modules[__package__].__dict__[mem] = _module
            index_views(_module, conf, env)
        except (ImportError, Exception), e:
            print `mem`, 'ImportError:', e
            continue


class View:
    
    __view__ = True
    __filters__ = True



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
# 
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
# 
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of posativ <info@posativ.org>.

VERSION = "0.2.1"
VERSION_SPLIT = tuple(VERSION.split('-')[0].split('.'))

import sys
reload(sys); sys.setdefaultencoding('utf-8')

import os, logging, locale, codecs
from optparse import OptionParser, make_option, OptionGroup
from jinja2 import Environment, FileSystemBytecodeCache

from acrylamid import defaults
from acrylamid import filters
from acrylamid import views
from acrylamid.utils import check_conf, yamllike, ColorFormatter, cache, \
                            ExtendedFileSystemLoader

log = logging.getLogger('acrylamid')
sys.path.insert(0, os.path.dirname(__package__))


class Acryl:
    """Main class for acrylamid functionality.  It handles initialization,
    defines default behavior, and also pushes the request through all
    steps until the output is rendered and we're complete."""
    
    def __init__(self):
        """Sets configuration and environment and creates the Request
        object"""
        
        usage = "usage: %prog [options] init\n" + '\n' \
                + "init     - initializes base structure\n" \
                + "gen      - render blog\n" \
                + "clean    - clean .cache/ and output/ dir"
#                + "srv (-p) - serving on port p (8000) and auto-rendering changes\n"
        
        options = [
            make_option("-v", "--verbose", action="store_const", dest="verbose",
                              help="debug information", const=logging.DEBUG),
            make_option("-q", "--quit", action="store_const", dest="verbose",
                              help="be silent (mostly)", const=logging.WARN,
                              default=logging.INFO),
            make_option("-f", "--force", action="store_true", dest="force",
                              help="force re-render", default=False),
            make_option("-c", "--config", dest="conf", help="alternate conf.yaml",
                              default="conf.yaml"),
            make_option("--version", action="store_true", dest="version",
                               help="print version details", default=False),                  
            ]
            
            
        parser = OptionParser(option_list=options, usage=usage)
        ext_group = OptionGroup(parser, "conf.yaml override")
        
        for key, value in yamllike(defaults.conf).iteritems():
            if key in ['views.', 'filters.']: continue
            ext_group.add_option('--'+key.replace('_', '-'), action="store",
                            dest=key, metavar=value, type=type(value) if type(value) != list else str)
                            
        parser.add_option_group(ext_group)
        (options, args) = parser.parse_args()
            
        console = logging.StreamHandler()
        console.setFormatter(ColorFormatter('%(message)s'))
        if options.verbose == logging.DEBUG:
            fmt = '%(msecs)d [%(levelname)s] %(name)s.py:%(lineno)s:%(funcName)s %(message)s'
            console.setFormatter(ColorFormatter(fmt))
        log = logging.getLogger('acrylamid')
        log.addHandler(console)
        log.setLevel(options.verbose)
        
        env = {'VERSION': VERSION, 'VERSION_SPLIT': VERSION_SPLIT}
        if options.version:
            print "acrylamid" + ' ' + env['VERSION']
            sys.exit(0)
        
        if len(args) < 1:
            parser.print_usage()
            sys.exit(1)
        
        # -- init -- #
        # TODO: acrylamid init --layout_dir=somedir to overwrite defaults
        
        if 'init' in args:
            if len(args) == 2:
                defaults.init(args[1], options.force)
            else:
                defaults.init(overwrite=options.force)
            sys.exit(0)
        
        conf = yamllike(defaults.conf)
        
        try:
            conf.update(yamllike(open(options.conf).read()))
        except OSError:
            log.critical('no config file found: %s. Try "acrylamid init".', options.conf)
            sys.exit(1)
        
        assert check_conf(conf)
                
        conf['acrylamid_name'] = "acrylamid"
        conf['acrylamid_version'] = env['VERSION']

        conf['output_dir'] = conf.get('output_dir', 'output/')
        conf['entries_dir'] = conf.get('entries_dir', 'content/')
        conf['layout_dir'] = conf.get('layout_dir', 'layouts/')
        
        conf.update(dict((k,v) for k,v in options.__dict__.iteritems() if v != None))

        # -- run -- #
        
        # clean .cache/ on --force and on `clean` to force re-rendering
        if options.force or args[0] in ['clean', 'rm']:
            try:
                for p in os.listdir('.cache/'):
                    os.remove(os.path.join('.cache/', p))
            except (OSError, IOError):
                pass
            if args[0] in ['clean', 'rm']: sys.exit(0)
                    
        if args[0] in ['gen', 'generate', 'render']:
            self.req = {'conf': conf, 'env': env, 'data': {}}
            self.run()
        
    def initialize(self, request):
        """Initializes Jinja2 environment, prepare locale and configure
        some minor things. Filter and View are inited and the user
        configuration is passed to.
        """
        
        conf = request['conf']
        env = request['env']
        
        # set up templating environment
        env['tt_env'] = Environment(loader=ExtendedFileSystemLoader(conf['layout_dir']),
                                    bytecode_cache=FileSystemBytecodeCache('.cache/'))

        # initialize the locale, will silently fail if locale is not
        # available and uses system's locale
        try:
            locale.setlocale(locale.LC_ALL, conf.get('lang', ''))
        except (locale.Error, TypeError):
            # invalid locale
            locale.setlocale(locale.LC_ALL, '')
            log.warn("unsupported locale '%s', set to '%s'", conf['lang'], locale.getlocale()[0])
        conf['lang'] = locale.getlocale()

        if not conf.has_key('www_root'):
            log.warn('no `www_root` specified, using localhost:8000')
            conf['www_root'] = 'http://localhost:8000/'
        
        env['protocol'] = conf['www_root'][0:conf['www_root'].find('://')]
        # take off the trailing slash for base_url
        if conf['www_root'].endswith("/"):
            conf['www_root'] = conf['www_root'][:-1]
        
        # check encoding is available
        try:
            codecs.lookup(conf['encoding'])
        except LookupError as e:
            log.fatal('no such encoding available: %s', conf['encoding'])
            sys.exit(1)
        
        # XXX implement _optional_ config argments like cache_dir
        # init to conf['cache_dir'] (defaults to '.cache/')
        cache.init()
        
        # import and initialize plugins
        filters.initialize(conf.get("ext_dir", []), request['conf'], request['env'],
                              exclude=conf.get("ext_ignore", []),
                              include=conf.get("ext_include", []))
        views.initialize(conf.get("ext_dir", []), request['conf'], request['env'])
        
        ns = filters.__dict__
        exec(''.join(conf['filters.'])) in ns
        ns = views.__dict__
        exec(''.join(conf['views.'])) in ns
        
    def run(self):
        """This will render everything.
        """
        request = self.req
        self.initialize(request)
        
        from acrylamid.core import handle
        from acrylamid.filters import get_filters, FilterList
        from acrylamid.views import get_views
        
        request = handle(request)
        
        filtersdict = get_filters()
        _views = get_views()
        
        for v in _views:
            log.debug(v)
            for i, entry in enumerate(request['entrylist']):
                if not v.__filters__: break
                
                log.debug(entry.filename)
                entryfilters = entry.get('filters', [])
                if isinstance(entryfilters, basestring):
                    entryfilters = [entryfilters]
                viewsfilters = views.filters+getattr(views, v.__module__).filters
                
                _filters = FilterList()
                for f in entryfilters+viewsfilters:
                    x,y = f.split('+')[:1][0], f.split('+')[1:]
                    if filtersdict[x] not in _filters:
                        _filters.append((x, filtersdict[x], y))
                
                entry.lazy_eval = _filters
            
            v(request)



from distutils.core import setup

"""acrylamid documentation
=======================

acrylamid is yet another lightweight static blogging software written in python
and designed to get a high quality output. Its licensed under BSD Style, 2 clauses.

Features
********

- blog articles, pages and rss/atom feeds
- theming support (using jinja2_)
- Markdown_ and reStructuredText_ as markup languages
- MathML generation using AsciiMathML_
- hyphenation using `&shy;`
- modern web-typography

.. _jinja2: http://jinja.pocoo.org/
.. _reStructuredText: http://docutils.sourceforge.net/rst.html
.. _Markdown: http://daringfireball.net/projects/markdown/
.. _AsciiMathML: http://www1.chapman.edu/~jipsen/mathml/asciimath.html

Quickstart
**********

::

    pip install acrylamid

You'll need ``python``, ``jinja2`` and either ``markdown`` (default) or
``docutils``. ``pygments`` and ``asciimathml`` for colored code listings
respectively MathML. Typography needs ``smartypants``. To get a full-featured
installation do:

::

    pip install docutils pygments asciimathml smartypants

Get acrylamid and edit *conf.yaml* and *layouts/*. Run acrylamid with:

::

    $> acrylamid init myblog
        create  myblog/content/
        ...
    $> cd myblog/
    $> acrylamid gen
          warn  using mtime from <fileentry f'content/sample entry.txt'>
        create  '/articles/index.html', written to output/articles/index.html
        create  'Die Verwandlung', written to output/2011/die-verwandlung/index.html
        create  '/atom/index.html', written to output/atom/index.html
        create  '/rss/index.html', written to output/rss/index.html
        create  '/', written to output/index.html

Using acrylamid
***************

- `conf.yaml <https://github.com/posativ/acrylamid/blob/master/docs/conf.yaml.rst>`_
- `filters <https://github.com/posativ/acrylamid/blob/master/docs/filters.rst>`_

Filters
**********

See `docs/filters.rst
<https://github.com/posativ/acrylamid/blob/master/docs/filters.rst>`_ for
detailed information. Currently supported by acrylamid:

- **markdown**: rendering Markdown (+asciimathml,pygments)
- **rest**: reStructuredText (+pygments)
- **typography**: https://code.google.com/p/typogrify/ (and custom modifications)
- **summarize**: summarizes posts to 200 words
- **hyphenation**: hyphenate words (len > 10) based on language
- **head_offset**: decrease headings by offset
- **pass-through**: don't render with Markdown or reStructuredText

Views
*****

- **articles**: articles overview
- **entry**: renders single entry to given slug
- **index**: creates pagination / and /page/:num
- **feeds**: valid atom/rss feed
- **tags**: sort by tag with pagination (/:tag and /:tag/:num)
"""

hyph_files = ['hyph-de-1996.chr.txt', 'hyph-en-us.chr.txt', 'hyph-fr.chr.txt',
    'hyph-de-1996.lic.txt', 'hyph-en-us.lic.txt', 'hyph-fr.lic.txt',
    'hyph-de-1996.pat.txt', 'hyph-en-us.pat.txt', 'hyph-fr.pat.txt']

setup(
    name='acrylamid',
    version='0.2.1',
    author='posativ',
    author_email='info@posativ.org',
    packages=['acrylamid', 'acrylamid.filters', 'acrylamid.views'],
    scripts=['bin/acrylamid'],
    package_data={'acrylamid.filters': ['hyph/*.txt']},
    url='http://pypi.python.org/pypi/acrylamid/',
    license='LICENSE.txt',
    description='yet another static blog generator',
    long_description=__doc__,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
    ],
    install_requires=[
        'Jinja2>=2.4',
        'Markdown>=2.0.1',
        'translitcodec>=0.2'
    ],
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import sys, os, re, logging
import codecs
import tempfile
from datetime import datetime
from os.path import join, exists, dirname, getmtime
from hashlib import md5

import traceback
from jinja2 import FileSystemLoader

try:
    import cPickle as pickle
except ImportError:
    import pickle

log = logging.getLogger('acrylamid.utils')
_slug_re = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.:]+')

try:
    import translitcodec
except ImportError:
    from unicodedata import normalize
    translitcodec = None
    log.debug("no 'translitcodec' found, using NFKD algorithm")


# Borrowed from werkzeug._internal
class _Missing(object):

    def __repr__(self):
        return 'no value'

    def __reduce__(self):
        return '_missing'


# Borrowed from werkzeug.utils
class cached_property(object):
    """A decorator that converts a function into a lazy property. The
    function wrapped is called the first time to retrieve the result
    and then that calculated result is used the next time you access
    the value::

    class Foo(object):

    @cached_property
    def foo(self):
    # calculate something important here
    return 42

    The class has to have a `__dict__` in order for this property to
    work.
    """

    # implementation detail: this property is implemented as non-data
    # descriptor. non-data descriptors are only invoked if there is
    # no entry with the same name in the instance's __dict__.
    # this allows us to completely get rid of the access function call
    # overhead. If one choses to invoke __get__ by hand the property
    # will still work as expected because the lookup logic is replicated
    # in __get__ for manual invocation.

    def __init__(self, func, name=None, doc=None):
        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func
        self._missing = _Missing()

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        value = obj.__dict__.get(self.__name__, self._missing)
        if value is self._missing:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value


class FileEntry:
    """This class gets it's data and metadata from the file specified
    by the filename argument"""
    
    __map__ = {'tag': 'tags', 'filter': 'filters'}
    __keys__ = ['permalink', 'filters', 'author', 'draft', 'tags', 'date', 'title', 'content', 'lang', 'description']
    
    title = content = ''
    draft = False
    permalink = lang = None
    tags = filters = lazy_eval = []
    
    def __init__(self, filename, encoding='utf-8'):
        """parsing FileEntry's YAML header.
        
        :param filename: path to open, plain text preferred
        :param encoding: reading content using this specific encoding codec."""
        
        self.mtime = os.path.getmtime(filename)
        self.date = datetime.fromtimestamp(self.mtime)
        self.filename = filename
        self.encoding = encoding
        self.parse()
        
    def __repr__(self):
        return "<fileentry f'%s'>" % self.filename
    
    @property
    def hash(self):
        if len(self.lazy_eval) == 0:
            return ''
        
        to_hash = []
        for t in self.lazy_eval:
            to_hash.append('%s:%.2f:%s' % (t[0], t[1].__priority__, t[2]))
        return '-'.join(to_hash) + self.filename
        
    @property
    def extension(self):
        return os.path.splitext(self.filename)[1][1:]
        
    @property
    def source(self):
        with codecs.open(self.filename, 'r', encoding=self.encoding, errors='replace') as f:
            return ''.join(f.readlines()[self._i:]).strip()
    
    @property
    def content(self):
       try:
            rv = cache.get(self.hash, mtime=self.mtime)
            if not rv:
                res = self.source
                for i, f, args in self.lazy_eval:
                    f.__dict__['__matched__'] = i
                    res = f(res, self, *args)
                rv = cache.set(self.hash, res)
            return rv
       except (IndexError, AttributeError):
           # jinja2 will ignore these Exceptions, better to catch them before
           traceback.print_exc(file=sys.stdout)
            
    @property
    def slug(self):
        return safeslug(self.title)
        
    @property
    def permalink(self):
        # TODO: fix hard-coded slug
        return expand('/:year/:slug/', self)

    @property
    def description(self):
        # TODO: this is really poor
        return self.source[:50].strip()+'...'
    
    @cached_property
    def has_changed(self):
        if not exists(cache._get_filename(self.hash)):
            self.content
            return True
        if getmtime(self.filename) > cache.get_mtime(self.hash):
            return True
        else:
            return False
    
    @property
    def draft(self):
        return True if self.get('draft', False) else False
        
    def get(self, key, default=None):
        return self.__dict__.get(key, default)
        
    def parse(self):
        """parsing yaml header and remember where content begins. Only append
        key,value if whitelisted in __keys__ and __map__ ."""
        
        meta = []; i = 0
        with codecs.open(self.filename, 'r', encoding=self.encoding, errors='replace') as f:
            while True:
                line = f.readline()
                i += 1
                if i == 1 and not line.strip():
                    break
                elif i == 1 and line.startswith('---'):
                    pass
                elif i > 1 and not line.startswith('---'):
                    meta.append(line)
                else:
                    break
                
        self._i = i
        for key, value in yamllike(''.join(meta)).iteritems():
            if key not in self.__keys__+self.__map__.keys():
                continue
            if isinstance(value, basestring):
                self.__dict__[key] = unicode(value.strip('"'))
            else:
                self.__dict__[key] = value

    def keys(self):
        return filter(lambda k: hasattr(self, k), self.__keys__)
        
    def __getitem__(self, key):
        """surjective dict. Return mapped key (tag -> tags) or raise KeyError."""
        if key in self.__map__:
            return self.__dict__[self.__map__[key]]
        elif key in self.__keys__:
            return getattr(self, key)
        else:
            raise KeyError("%s has no such attribute '%s'" % (self, key))


class ColorFormatter(logging.Formatter):
    """Implements basic colored output using ANSI escape codes."""

    # $color + BOLD
    BLACK = '\033[1;30m%s\033[0m'
    RED = '\033[1;31m%s\033[0m'
    GREEN = '\033[1;32m%s\033[0m'
    YELLOW = '\033[1;33m%s\033[0m'
    GREY = '\033[1;37m%s\033[0m'
    RED_UNDERLINE = '\033[4;31m%s\033[0m'

    def __init__(self, fmt='[%(levelname)s] %(name)s: %(message)s', debug=False):
        logging.Formatter.__init__(self, fmt)
        self.debug = debug

    def format(self, record):

        keywords = {'skip': self.BLACK, 'create': self.GREEN, 'identical': self.BLACK,
                    'update': self.YELLOW, 'changed': self.YELLOW}
                    
        if record.levelno == logging.INFO:
            for item in keywords:
                if record.msg.startswith(item):
                    record.msg = record.msg.replace(item, ' '*2 + \
                                    keywords[item] % item.rjust(8))
        elif record.levelno >= logging.WARN:
            record.levelname = record.levelname.replace('WARNING', 'WARN')
            record.msg = ''.join([' '*2, self.RED % record.levelname.lower().rjust(8),
                                  '  ', record.msg])

        return logging.Formatter.format(self, record)


class ExtendedFileSystemLoader(FileSystemLoader):

    def load(self, environment, name, globals=None):
        """patched `load` to add a has_changed property"""
        code = None
        if globals is None:
            globals = {}

        source, filename, uptodate = self.get_source(environment, name)

        bcc = environment.bytecode_cache
        if bcc is not None:
            bucket = bcc.get_bucket(environment, name, filename, source)
            p = bcc._get_cache_filename(bucket)
            has_changed = getmtime(filename) > getmtime(p) if exists(p) else False
            code = bucket.code

        if code is None:
            code = environment.compile(source, name, filename)

        if bcc is not None and bucket.code is None:
            bucket.code = code
            bcc.set_bucket(bucket)

        tt = environment.template_class.from_code(environment, code, globals, uptodate)
        tt.has_changed = has_changed
        return tt


def check_conf(conf):
    """Rudimentary conf checking.  Currently every *_dir except
    `ext_dir` (it's a list of dirs) is checked wether it exists."""

    # directories

    for key, value in conf.iteritems():
        if key.endswith('_dir') and not key in ['ext_dir', ]:
            if os.path.exists(value):
                if os.path.isdir(value):
                    pass
                else:
                    log.error("'%s' must be a directory" % value)
                    sys.exit(1)
            else:
                os.mkdir(value)
                log.warning('%s created...' % value)

    return True
    
def yamllike(conf):
    """pyyaml replacement (not really yet, but works for me). Parsing
    first-layer YAML (okay: key,value assignments) into a python dict.
    yamllike is filter. and view. aware, that means all assignments starting
    with this string will exec into the given __init__ environment."""
    
    conf = [line.strip() for line in conf.split('\n')
                if not line.startswith('#') and line.strip()]
    
    config = {}
    config['views.'] = []
    config['filters.'] = []
    for line in conf:
        try:
            key, value = [x.strip() for x in line.split(':', 1)]
        except ValueError:
            # do something
            log.warn('conf.yaml -> ValueError: %s' % line)
            continue
        
        if key.startswith('filters.'):
            config['filters.'].append(key.replace('filters.', '')+' = '+value+'\n')
        elif key.startswith('views.'):
            config['views.'].append(key.replace('views.', '')+' = '+value+'\n')
        else:
            if value == '':
                config[key] = None
            elif value.isdigit():
                config[key] = int(value)
            elif value.lower() in ['true', 'false']:
                 config[key] = True if value.capitalize() == 'True' else False
            elif value[0] == '[' and value[-1] == ']':
                config[key] = list([unicode(x.strip()) for x in value[1:-1].split(',') if x.strip()])
            else:
                config[key] = value
    
    return config


def render(tt, *dicts, **kvalue):
    """helper function to merge multiple dicts and additional key=val params
    to a single environment dict used by jinja2 templating. Note, merging will
    first update dicts in given order, then (possible) overwrite single keys
    in kvalue."""

    env = {}
    for d in dicts:
        env.update(d)
    for key in kvalue:
        env[key] = kvalue[key]
    
    return tt.render(env)


def mkfile(content, path, message, force=False):
    """Creates entry in filesystem. Overwrite only if content differs.

    :param content: rendered html/xml to write
    :param entry: FileEntry object
    :param path: path to write to
    :param force: force overwrite, even nothing has changed (defaults to `False`)"""

    if exists(dirname(path)) and exists(path):
        with file(path) as f:
            old = f.read()
        if content == old and not force:
            event.skip(message)
        else:
            with open(path, 'w') as f:
                f.write(content)
            event.changed(message)
    else:
        try:
            os.makedirs(dirname(path))
        except OSError:
            # dir already exists (mostly)
            pass
        with open(path, 'w') as f:
            f.write(content)
        event.create(message, path)

    
def expand(url, entry):
    """expanding '/:year/:slug/' scheme into e.g. '/2011/awesome-title/"""
    m = {':year': str(entry.date.year), ':month': str(entry.date.month),
         ':day': str(entry.date.day), ':slug': entry.slug}
    
    for val in m:
        url = url.replace(val, m[val])
    return url


def joinurl(*args):
    """joins multiple urls to one single domain without loosing root (first element)"""
    
    r = []
    for i, mem in enumerate(args):
        if i > 0:
            mem = str(mem).lstrip('/')
        r.append(mem)
    return join(*r)


def safeslug(slug):
    """Generates an ASCII-only slug.  Borrowed from
    http://flask.pocoo.org/snippets/5/"""

    result = []
    for word in _slug_re.split(slug.lower()):
        if translitcodec:
            word = word.encode('translit/long')
        else:
            word = normalize('NFKD', word).encode('ascii', 'ignore')
        if word and not word[0] in '-:':
            result.append(word)
    return unicode('-'.join(result))


def paginate(list, ipp, func=lambda x: x):
    list = filter(func, list)
    return (list[x*ipp:(x+1)*ipp] for x in range(len(list)/ipp+1))


class cache(object):
    """A cache that stores the entries on the file system.  Borrowed from
    werkzeug.contrib.cache, see their AUTHORS and LICENSE for additional
    copyright information.
    
    cache is designed as singleton and should not constructed using __init__ .
    >>> cache.init('.mycache/')
    >>> cache.get(key, default=None, mtime=0.0)
    >>> cache.set(key, value)

    :param cache_dir: the directory where cache files are stored.
    :param mode: the file mode wanted for the cache files, default 0600
    """
    
    _fs_transaction_suffix = '.__ac_cache'
    cache_dir = '.cache/'
    mode = 0600
    
    @classmethod
    def _get_filename(self, key):
        hash = md5(key).hexdigest()
        return os.path.join(self.cache_dir, hash)
    
    @classmethod
    def _list_dir(self):
        """return a list of (fully qualified) cache filenames"""
        return [os.path.join(self.cache_dir, fn) for fn in os.listdir(self.cache_dir)
                if not fn.endswith(self._fs_transaction_suffix) \
                   and not fn.endswith('.cache')]
    
    @classmethod
    def init(self, cache_dir=None, mode=0600):
        if cache_dir:
            self.cache_dir = cache_dir
        if mode:
            self.mode = mode
        if not exists(self.cache_dir):
            try:
                os.mkdir(self.cache_dir, 0700)
            except OSError:
                log.fatal("could not create directory '%s'" % self.cache_dir)
                sys.exit(1)

    @classmethod
    def clear(self):
        for fname in self._list_dir():
            try:
                os.remove(fname)
            except (IOError, OSError):
                pass

    @classmethod
    def get(self, key, default=None, mtime=0.0):
        try:
            filename = self._get_filename(key)
            if mtime > getmtime(filename):
                return default
            with open(filename, 'rb') as fp:
                return pickle.load(fp)
            os.remove(filename)
        except (OSError, IOError, pickle.PickleError):
            pass
        return default

    @classmethod
    def set(self, key, value):
        filename = self._get_filename(key)
        try:
            fd, tmp = tempfile.mkstemp(suffix=self._fs_transaction_suffix,
                                       dir=self.cache_dir)
            with os.fdopen(fd, 'wb') as fp:
                pickle.dump(value, fp, pickle.HIGHEST_PROTOCOL)
            os.rename(tmp, filename)
            os.chmod(filename, self.mode)
        except (IOError, OSError):
            pass
        
        return value
    
    @classmethod
    def get_mtime(self, key, default=0.0):
        filename = self._get_filename(key)
        try:
            mtime = getmtime(filename)
        except (OSError, IOError):
            return default
        return mtime


class event:
    
    @classmethod
    def __init__(self):
        pass
    
    @classmethod
    def create(self, what, path):
        log.info("create  '%s', written to %s", what, path)
    
    @classmethod
    def changed(self, what):
        log.info("changed  content of '%s'", what)
    
    @classmethod
    def skip(self, what):
        log.info("skip  '%s' is up to date", what)



#!/usr/bin/env python
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

from acrylamid.filters import Filter
from acrylamid.filters import log

class Markdown(Filter):
    
    __name__ = 'Pass-Through'
    __match__ = ['pass', 'plain', 'html', 'xhtml']
    __conflicts__ = ['rst', 'md']
    __priority__ = 70.0
    
    def __init__(self, conf, env):
        pass
        
    def __call__(self, content, request, *filters):
        return content



# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

from acrylamid.filters import Filter

from HTMLParser import HTMLParser
from cgi import escape

class Summarizer(HTMLParser):

    def __init__(self, text, href, maxwords=100):
        HTMLParser.__init__(self)
        self.maxwords = maxwords
        self.href = href
        self.summarized = ''
        self.words = 0
        self.stack = []

        self.feed(text)

    def handle_starttag(self, tag, attrs):
        '''Apply and stack each read tag until we reach maxword.'''

        def tagify(tag, attrs):
            '''convert parsed tag back into a html tag'''
            if attrs:
                return '<%s %s>' % (tag, ' '.join(['%s="%s"' % (k, escape(v))
                                        for k,v in attrs]))
            else:
                return '<%s>' % tag

        if self.words < self.maxwords:
            self.stack.append(tag)
            self.summarized += tagify(tag, attrs)

    def handle_data(self, data):

        if self.words >= self.maxwords:
            pass
        else:
            words = data.split(' ')
            if self.words + len(words) < self.maxwords:
                '''if the next few words will not go over maxwords'''
                self.words += len(words)
                self.summarized += data
            else:
                '''we can put some words before we reach the word limit'''
                if 'a' not in self.stack and self.stack and self.stack[-1] != 'ul':
                    somewords = self.maxwords - self.words
                    self.words += somewords
                    self.summarized += ' '.join(words[:somewords]) + ' '
                    self.summarized += '<span>&#8230;<a href="%s" class="continue">weiterlesen</a>.</span>' % self.href
                else:
                    self.maxwords += len(words)
                    self.words += len(words)
                    self.summarized += ' '.join(words)

    def handle_endtag(self, tag):
        '''Until we reach not the maxwords limit, we can safely pop every ending tag,
           added by handle_starttag. Afterwards, we apply missing endings tags if missing.'''
        if self.words < self.maxwords:
            self.stack.pop()
            self.summarized += '</%s>' % tag
        else:
            if self.stack:
                for x in range(len(self.stack)):
                    self.summarized += '</%s>' % self.stack.pop()
                    
    def handle_entityref(self, entity):
        '''handle &shy; correctly'''
        if self.words < self.maxwords:
            self.summarized += '&'+ entity + ';'
            
    def handle_charref(self, char):
        if self.words < self.maxwords:
            self.summarized += '&#' + char + ';'


class Summarize(Filter):
    
    __name__ = 'Summarize'
    __match__ = ['summarize', 'sum']
   
    def __init__(self, conf, env):
        self.env = env
        
    def __call__(self, content, req):
        
        return Summarizer(content, req.permalink, 200).summarized
